/* 

  ****************   NO WARRANTY  *****************

Since the Aspirin/MIGRAINES system is licensed free of charge,
the MITRE Corporation provides absolutley no warranty. Should
the Aspirin/MIGRAINES system prove defective, you must assume
the cost of all necessary servicing, repair or correction.
In no way will the MITRE Corporation be liable to you for
damages, including any lost profits, lost monies, or other
special, incidental or consequential damages arising out of
the use or inability to use the Aspirin/MIGRAINES system.

  *****************   COPYRIGHT  *******************

This software is the copyright of The MITRE Corporation. 
It may be freely used and modified for research and development
purposes. We require a brief acknowledgement in any research
paper or other publication where this software has made a significant
contribution. If you wish to use it for commercial gain you must contact 
The MITRE Corporation for conditions of use. The MITRE Corporation 
provides absolutely NO WARRANTY for this software.

   January, 1992 
   Russell Leighton
   The MITRE Corporation
   7525 Colshire Dr.
   McLean, Va. 22102-3481

*/
/* type4 file reader: Type4 files are floating point files of data patterns.
                      The format is:
                         data pattern  
                         data pattern  
                         data pattern  
			      ...etc.
                         target pattern  
                         target pattern  
                         target pattern  
			      ...etc.
		      It is assumed that the size and
		      dimensions of these patterns correspond to the size
		      and dimensions of the input to ALL black boxes.
 */

#include "BpDatafile.h"

void _type4_reader(file_descriptor, datafile, data)
     int file_descriptor;                 /* open file */
     DATAFILEPTR datafile;                /* this file */
     DATAPTR data;                        /* root structure */
{
  register int counter;
  int file_nfloats, file_nbytes;
  int npatterns;
  int input_size = (data->inputs_xdim * data->inputs_ydim);
  int target_size = (data->targets_xdim * data->targets_ydim);
  float *data_block;
  float *target_block;
  
  
  /* get size of the file */
  file_nbytes = am_file_size(datafile->filename);
  file_nfloats = ((int)file_nbytes)/sizeof(float);  /* # floats in file */
  
  /* derive the number of patterns */
  npatterns = file_nfloats/ (input_size + target_size);
  
  if (datafile->scroll_step) { /* if scrolling */
    fprintf(stderr,"\nNo scrolling on Type4 files (%s)",
	    datafile->filename);
    return;
  } else { /* not scrolling */
    if (file_nfloats % (input_size + target_size) != 0) { /* error check */
      fprintf(stderr,"\n%s has NON integral number of patterns!\n",
	      datafile->filename);
      return;
    }/* end if */
    datafile->npatterns = npatterns;
    if (datafile->switch_cycle < 0) datafile->switch_cycle = npatterns; /* (switch: all) */
    datafile->size = input_size;
    /* read the file into an array of the same size */
    data_block=(float *)am_alloc_mem(input_size*npatterns*sizeof(float));
    /* make pointer vectors */
    datafile->inputs = (float **)am_alloc_mem(npatterns * sizeof(float *));
    datafile->targets = (float **)am_alloc_mem(npatterns * sizeof(float *));
  }/* end else */
  
  datafile->xdim = data->inputs_xdim;
  datafile->ydim = data->inputs_ydim;
  
  target_block=(float *)am_alloc_mem(target_size*npatterns*sizeof(float));
  
  /* read patterns  */
  am_read(file_descriptor,
       (char *)data_block,
       input_size*npatterns*sizeof(float));

  /* read targets  */
  am_read(file_descriptor,
       (char *)target_block,
       target_size*npatterns*sizeof(float));
  
  /* update pointers into the inputs array */
  for (counter=0; counter<npatterns; counter++) {
    datafile->inputs[counter] = data_block + (counter * input_size);
    datafile->targets[counter] = target_block + (counter * target_size);
  }/* end for */
  
}/* end _type4_reader */
